/*
   BrwMon1.prg
   Free, open source code to use with Harbour/miniGUI 2.6.7
   by Jos Arturo Garca Vzquez
   arturogarciavazquez@yahoo.com.mx  -  write "HMGforum - your text" in subject
   
   Simulate OnChange event for right click in a BROWSE control.

   While this code is usable, it always will be considered Beta.
   Provisional workaround, keep looking for news of miniGUI official  2.6.7 ->
   YOU *MUST* READ   BRWMON1.TXT   BEFORE USE THIS ROUTINES
   
   Quick guide:
   1. #include "BRWMON1.PRG"     <-- to insert this routines in your code
   2. BRWMonitorBegin( "Form_1", ;
                {  {"Form_1", "Browse_1", "dbfAlias1"} , ;
                   {"Form_1", "Browse_2", "dbfAlias2"}...} , "MyFunction", 500 )
   3. Function MyFunction ( arraylistWithStatus , samearrayYouProvided ) --> .T.
*/

#define cBRWMonitorTimerName "BRWMonitorTimerName"

//------------------------------------------------------------------------------
Function BRWMonitor( pnCodeToSet )		// avaliable only for debug purposes
LOCAL nCodeToReturn

	IF .NOT. TYPE("nBRWPublicCode") == "N"
		PUBLIC nBRWPublicCode := 1 		// default first status code is:  1 (ON)
	ENDIF
	
	nCodeToReturn := nBRWPublicCode		// allways returns previous status

	IF PCOUNT() = 1
		nBRWPublicCode := pnCodeToSet		// optional, new code to set
	ENDIF

Return nCodeToReturn


//------------------------------------------------------------------------------
Function BRWMonitorBegin( pcWindowNameToUse, pacBrwObjectsToMonitor, pcProcedureName, pnInterval )
LOCAL nInterval := IIF( pnInterval < 100, 100, INT(pnInterval) ) // dont be agressive

	IF BRWMonitor() = 0
		Return Nil
	ENDIF
	
	DEFINE TIMER &cBRWMonitorTimerName OF &pcWindowNameToUse ;
		INTERVAL nInterval ;
		ACTION BRWMonitorTimerTask( pcWindowNameToUse, pacBrwObjectsToMonitor, pcProcedureName )

Return Nil


//------------------------------------------------------------------------------
Function BRWMonitorTimerPause( pcWindowNameToUse )
	IF BRWMonitor() = 0
		Return Nil
	ENDIF
	
	SetProperty( pcWindowNameToUse, cBRWMonitorTimerName, "ENABLED", .F. )
Return Nil


//------------------------------------------------------------------------------
Function BRWMonitorTimerResume( pcWindowNameToUse )
	IF BRWMonitor() = 0
		Return Nil
	ENDIF
	
	SetProperty( pcWindowNameToUse, cBRWMonitorTimerName, "ENABLED", .T. )
Return Nil


//------------------------------------------------------------------------------
Function BRWMonitorTimerRelease( pcWindowNameToUse )
	IF BRWMonitor() = 0
		Return Nil
	ENDIF
	
	DoMethod( pcWindowNameToUse, cBRWMonitorTimerName, "RELEASE" )
Return Nil


//------------------------------------------------------------------------------
// internal monitoring task, programer not need to call this function
Function BRWMonitorTimerTask( pcWindowNameToUse, pacBrwObjectsToMonitor, pcProcedureName )
LOCAL n, ;
		nCurrentValueFromBrwObject, ;
		nCurrentValueFromRecno, ;
		cRecnoExpression, ;
		lUDFNeedToBeCalled, ;
		lBrwWantToConinue, ;
		cCallUDFwithStatusList
PRIVATE alBRWList := {}		// seems to be necesary PRIVATE to use with macro &
PRIVATE acBrwObjectsToMonitor := pacBrwObjectsToMonitor

	IF BRWMonitor() = 0
		Return Nil
	ENDIF
	
	BRWMonitorTimerPause( pcWindowNameToUse )	// avoid recursive timer calls
	
	// let's compare "Form_name"."Browse_name".VALUE  !=  "dbfAliasName"->( RECNO() )
	// and stores each result in alBRWList array
	lUDFNeedToBeCalled := .F.
	FOR n := 1 TO len(pacBrwObjectsToMonitor)
		nCurrentValueFromBrwObject := getproperty( pacBrwObjectsToMonitor[n,1], pacBrwObjectsToMonitor[n,2], "VALUE" )
		cRecnoExpression := pacBrwObjectsToMonitor[n,3] + "->( RECNO() )"
		nCurrentValueFromRecno := &cRecnoExpression
		aAdd( alBRWList, ( nCurrentValueFromBrwObject != nCurrentValueFromRecno ) )
		IF alBRWList[n] = .T. 	// pointers changed ? (most probably by right click)
			lUDFNeedToBeCalled := .T.
		ENDIF
	NEXT n

	lBrwWantToConinue := .T.
	IF lUDFNeedToBeCalled	// let us call user defined function
		// User defined function *MUST* return .T./.F. if want (or not) resume monitoring
		cCallUDFwithStatusList := pcProcedureName + "( alBRWList, acBrwObjectsToMonitor )"
		lBrwWantToConinue := &cCallUDFwithStatusList
	ENDIF
	
	IF lBrwWantToConinue
		BRWMonitorTimerResume( pcWindowNameToUse )	// resume timer "monitoring"
	ENDIF
	
Return Nil

// EOF
